import numpy as np
from .makeStruc import calcDistance
from .isOverlap import isOverlapMolecule, isOverlapAtom

def placeMols(shape, og, density, tol, shapeType, radii):
    '''Place molecules in the grid defined by the density'''
    gridPoints = calcDistance(shape, og, density, shapeType)

    def calcCentroid(coords):
        """Calculate the geometric centroid of a set of coordinates."""
        return np.mean(coords, axis=0)

    # Tanslate molecules such that their centroid is at the center
    # of the target point
    def translateMol(coords, target):
        '''Translate molecules to a new grid point'''
        atomLabels = [atom[0] for atom in coords]
        pdbInfo = [atom[4] if len(atom) == 5 else None for atom in coords]
        xyzCoords = np.array([atom[1:4] for atom in coords])

        centroid = calcCentroid(xyzCoords)
        transVec = np.array(target) - centroid
        translatedXYZ = xyzCoords + transVec

        transMol = [[label] + coord.tolist() + ([info] if info is not None else [])
                    for label, coord, info in zip(atomLabels, translatedXYZ, pdbInfo)]
        
        return transMol

    
    mols = []
    for point in gridPoints:
        newPoint = translateMol(og, point)

        if len(og) == 1:
            if not isOverlapAtom(newPoint, mols, radii, tol):
                mols.append(newPoint)
                type = "atom"
        else:
            if not isOverlapMolecule(newPoint, mols, radii, tol):
                mols.append(newPoint)
                type = "molecule"
    
    return mols, type



# Top level package
from .Box3D import Box3d, drawBox
from .defaultParams import defaultParams
from .drawMol import drawMolBox, drawMolSphere
from .isOverlap import isOverlapAtom, isOverlapMolecule
from .makeStruc import makeBase, calcCenter, reCenter, shiftPoints, randReorient, calcDensity, calcNumMol
from .readWriteFiles import getInput, readStrucFile, readPdb, writeOutput, writePdb, writeXYZ, getElementData
from .setAtomProp import setAtomicMass, setAtomicRadius
from .shiftBox import atomsFillBox, atomsRandBox, moleculesFillBox, moleculesRandBox, inBox
from .shiftSphere import atomFillSphere, atomRandSphere, moleculeFillSphere, moleculeRandSphere
from .Sphere3D import Sphere3d
from .shiftDensity import placeMols

from . import _version
__version__ = _version.get_versions()['version']



'''
This module checks if atoms within a molecule overlaps
or if two different molecules overlap
'''

import math

def isOverlapMolecule(newAtom, filledAtoms, radii, tol):
    '''Checks if two molecules overlap'''
    for molecule in filledAtoms:
        for atom1, atom2 in zip(newAtom, molecule):
            distance = math.sqrt(
                (atom1[1] - atom2[1])**2 +
                (atom1[2] - atom2[2])**2 +
                (atom1[3] - atom2[3])**2
            )

            if distance < (radii[atom1[0]] + radii[atom2[0]]) + tol:
                return True
    return False

def isOverlapAtom(newAtom, filledAtoms, radii, tol):
    '''Checks if two atoms overlap'''
    for atom in filledAtoms:
        distance = math.sqrt(
            (newAtom[1] - atom[1])**2 +
            (newAtom[2] - atom[2])**2 +
            (newAtom[3] - atom[3])**2
        )

        if distance < (radii[newAtom[0]] + radii[atom[0]]) + tol:
            return True
    return False
